@@ -656,6 +656,9 @@ namespace nux
         m_GLCtx,
         1, 0, false);
 
+    if (m_DeviceFactory->GetGpuInfo().Support_EXT_Framebuffer_Object())
+      m_DeviceFactory->GetFrameBufferObject()->SetupFrameBufferObject();
+
     m_GraphicsContext = new GraphicsEngine(*this);
 
     //EnableVSyncSwapControl();
@@ -711,6 +714,10 @@ namespace nux
         _fb_config,
         m_GLCtx,
         1, 0, false);
+
+    if (m_DeviceFactory->GetGpuInfo().Support_EXT_Framebuffer_Object())
+      m_DeviceFactory->GetFrameBufferObject()->SetupFrameBufferObject();
+
     m_GraphicsContext = new GraphicsEngine(*this);
 
     InitGlobalGrabWindow();
@@ -1680,6 +1687,7 @@ namespace nux
         m_pEvent->x_root = 0;
         m_pEvent->y_root = 0;
         m_pEvent->key_modifiers = GetModifierKeyState(xevent.xkey.state);
+        m_pEvent->x11_timestamp = xevent.xbutton.time;
         MousePress(xevent, m_pEvent);
         //nuxDebugMsg("[GraphicsDisplay::ProcessXEvents]: ButtonPress event.");
         break;
